import { DeleteResult, EntityManager, InsertResult } from "typeorm";
import { dataSource } from "../../../data-source";
import { webapiPermission } from "../../../entity/user/webapi_permission";
import InternalException from "../../../exceptions/internalException";
import WebapiService from "../../../service/user/webapiService";
import {
  CreateWebapiPermissionCommand,
  DeleteWebapiPermissionCommand,
  UpdateWebapiPermissionsCommand,
} from "./webapiPermissionCommand";
import PermissionHelper from "../../../helpers/permissionHelper";
import WebapiPermissionService from "../../../service/user/webapiPermissionService";
import { PermissionString } from "../../../type/permissionTypes";

export default abstract class WebapiPermissionCommandHandler {
  /**
   * @description update api permissions
   * @param {UpdateWebapiPermissionsCommand} updateWebapiPermissions
   * @returns {Promise<void>}
   */
  static async updatePermissions(updateWebapiPermissions: UpdateWebapiPermissionsCommand): Promise<void> {
    let currentPermissions = (await WebapiPermissionService.getByApi(updateWebapiPermissions.webapiId)).map(
      (r) => r.permission
    );
    return await dataSource.manager
      .transaction(async (manager) => {
        let newPermissions = PermissionHelper.getWhatToAdd(currentPermissions, updateWebapiPermissions.permissions);
        let removePermissions = PermissionHelper.getWhatToRemove(
          currentPermissions,
          updateWebapiPermissions.permissions
        );
        if (newPermissions.length != 0) {
          await this.updatePermissionsAdd(manager, updateWebapiPermissions.webapiId, newPermissions);
        }
        if (removePermissions.length != 0) {
          await this.updatePermissionsRemove(manager, updateWebapiPermissions.webapiId, removePermissions);
        }
      })
      .then(() => {})
      .catch((err) => {
        throw new InternalException("Failed saving api permissions", err);
      });
  }

  private static async updatePermissionsAdd(
    manager: EntityManager,
    webapiId: number,
    permissions: Array<PermissionString>
  ): Promise<InsertResult> {
    return await manager
      .createQueryBuilder()
      .insert()
      .into(webapiPermission)
      .values(
        permissions.map((p) => ({
          permission: p,
          webapiId: webapiId,
        }))
      )
      .orIgnore()
      .execute();
  }

  private static async updatePermissionsRemove(
    manager: EntityManager,
    webapiId: number,
    permissions: Array<PermissionString>
  ): Promise<DeleteResult> {
    return await manager
      .createQueryBuilder()
      .delete()
      .from(webapiPermission)
      .where("webapiId = :id", { id: webapiId })
      .andWhere("permission IN (:...permission)", { permission: permissions })
      .execute();
  }

  /**
   * @description grant permission to user
   * @param {CreateWebapiPermissionCommand} createPermission
   * @returns {Promise<number>}
   */
  static async create(createPermission: CreateWebapiPermissionCommand): Promise<number> {
    return await dataSource
      .createQueryBuilder()
      .insert()
      .into(webapiPermission)
      .values({
        permission: createPermission.permission,
        webapiId: createPermission.webapiId,
      })
      .execute()
      .then((result) => {
        return result.identifiers[0].id;
      })
      .catch((err) => {
        throw new InternalException("Failed saving api permission", err);
      });
  }

  /**
   * @description remove permission from api
   * @param {DeleteWebapiPermissionCommand} deletePermission
   * @returns {Promise<any>}
   */
  static async delete(deletePermission: DeleteWebapiPermissionCommand): Promise<any> {
    return await dataSource
      .createQueryBuilder()
      .delete()
      .from(webapiPermission)
      .where("webapiId = :id", { id: deletePermission.webapiId })
      .andWhere("permission = :permission", { permission: deletePermission.permission })
      .execute()
      .then(() => {})
      .catch((err) => {
        throw new InternalException("failed api permission removal", err);
      });
  }
}
